# Copyright 2014 Mirantis, Inc.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from unittest import mock

from mistralclient.api.v2 import cron_triggers
from mistralclient.commands.v2 import cron_triggers as cron_triggers_cmd
from mistralclient.tests.unit import base


TRIGGER_DICT = {
    'name': 'my_trigger',
    'workflow_name': 'flow1',
    'workflow_input': {},
    'workflow_params': {},
    'pattern': '* * * * *',
    'next_execution_time': '4242-12-20 13:37',
    'remaining_executions': 5,
    'created_at': '1',
    'updated_at': '1'
}

TRIGGER = cron_triggers.CronTrigger(mock, TRIGGER_DICT)


class TestCLITriggersV2(base.BaseCommandTest):
    @mock.patch('mistralclient.commands.v2.cron_triggers.Create.'
                '_convert_time_string_to_utc')
    @mock.patch('argparse.open', create=True)
    def test_create(self, mock_open, mock_convert):
        self.client.cron_triggers.create.return_value = TRIGGER
        mock_open.return_value = mock.MagicMock(spec=open)

        result = self.call(
            cron_triggers_cmd.Create,
            app_args=['my_trigger', 'flow1', '--pattern', '* * * * *',
                      '--params', '{}', '--count', '5', '--first-time',
                      '4242-12-20 13:37', '--utc']
        )

        mock_convert.assert_not_called()
        self.assertEqual(
            (
                'my_trigger', 'flow1', {}, '* * * * *',
                '4242-12-20 13:37', 5, '1', '1'
            ),
            result[1]
        )

    @mock.patch('mistralclient.commands.v2.cron_triggers.Create.'
                '_convert_time_string_to_utc')
    @mock.patch('argparse.open', create=True)
    def test_create_no_utc(self, mock_open, mock_convert):
        self.client.cron_triggers.create.return_value = TRIGGER
        mock_open.return_value = mock.MagicMock(spec=open)
        mock_convert.return_value = '4242-12-20 18:37'

        result = self.call(
            cron_triggers_cmd.Create,
            app_args=['my_trigger', 'flow1', '--pattern', '* * * * *',
                      '--params', '{}', '--count', '5', '--first-time',
                      '4242-12-20 13:37']
        )

        mock_convert.assert_called_once_with('4242-12-20 13:37')
        self.client.cron_triggers.create.assert_called_once_with(
            'my_trigger', 'flow1', {}, {}, '* * * * *', '4242-12-20 18:37', 5)
        self.assertEqual(
            (
                'my_trigger', 'flow1', {}, '* * * * *',
                '4242-12-20 13:37', 5, '1', '1'
            ),
            result[1]
        )

    @mock.patch('mistralclient.commands.v2.cron_triggers.time')
    def test_convert_time_string_to_utc_from_utc(self, mock_time):
        cmd = cron_triggers_cmd.Create(self.app, None)

        mock_time.daylight = 0
        mock_time.altzone = 0
        mock_time.timezone = 0
        mock_localtime = mock.Mock()
        mock_localtime.tm_isdst = 0
        mock_time.localtime.return_value = mock_localtime

        utc_value = cmd._convert_time_string_to_utc('4242-12-20 13:37')

        expected_time = '4242-12-20 13:37'

        self.assertEqual(expected_time, utc_value)

    @mock.patch('mistralclient.commands.v2.cron_triggers.time')
    def test_convert_time_string_to_utc_from_dst(self, mock_time):
        cmd = cron_triggers_cmd.Create(self.app, None)

        mock_time.daylight = 1
        mock_time.altzone = (4 * 60 * 60)
        mock_time.timezone = (5 * 60 * 60)
        mock_localtime = mock.Mock()
        mock_localtime.tm_isdst = 1
        mock_time.localtime.return_value = mock_localtime

        utc_value = cmd._convert_time_string_to_utc('4242-12-20 13:37')

        expected_time = '4242-12-20 17:37'

        self.assertEqual(expected_time, utc_value)

    @mock.patch('mistralclient.commands.v2.cron_triggers.time')
    def test_convert_time_string_to_utc_no_dst(self, mock_time):
        cmd = cron_triggers_cmd.Create(self.app, None)

        mock_time.daylight = 1
        mock_time.altzone = (4 * 60 * 60)
        mock_time.timezone = (5 * 60 * 60)
        mock_localtime = mock.Mock()
        mock_localtime.tm_isdst = 0
        mock_time.localtime.return_value = mock_localtime

        utc_value = cmd._convert_time_string_to_utc('4242-12-20 13:37')

        expected_time = '4242-12-20 18:37'

        self.assertEqual(expected_time, utc_value)

    def test_list(self):
        self.client.cron_triggers.list.return_value = [TRIGGER]

        result = self.call(cron_triggers_cmd.List)

        self.assertEqual(
            [(
                'my_trigger', 'flow1', {}, '* * * * *',
                '4242-12-20 13:37', 5, '1', '1'
            )],
            result[1]
        )

    def test_get(self):
        self.client.cron_triggers.get.return_value = TRIGGER

        result = self.call(cron_triggers_cmd.Get, app_args=['name'])

        self.assertEqual(
            (
                'my_trigger', 'flow1', {}, '* * * * *',
                '4242-12-20 13:37', 5, '1', '1'
            ),
            result[1]
        )

    def test_delete(self):
        self.call(cron_triggers_cmd.Delete, app_args=['name'])

        self.client.cron_triggers.delete.assert_called_once_with('name')

    def test_delete_with_multi_names(self):
        self.call(cron_triggers_cmd.Delete, app_args=['name1', 'name2'])

        self.assertEqual(2, self.client.cron_triggers.delete.call_count)
        self.assertEqual(
            [mock.call('name1'), mock.call('name2')],
            self.client.cron_triggers.delete.call_args_list
        )



# Copyright 2014 - Mirantis, Inc.
# Copyright 2015 - StackStorm, Inc.
# Copyright 2016 - Brocade Communications Systems, Inc.
# Copyright 2020 - Nokia Software.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import pkg_resources as pkg
from unittest import mock

from oslo_serialization import jsonutils

from mistralclient.api.v2 import executions
from mistralclient.commands.v2 import executions as execution_cmd
from mistralclient.tests.unit import base

EXEC_DICT = {
    'id': '123',
    'workflow_id': '123e4567-e89b-12d3-a456-426655440000',
    'workflow_name': 'some',
    'workflow_namespace': '',
    'root_execution_id': '',
    'description': '',
    'state': 'SUCCESS',
    'state_info': None,
    'created_at': '2020-02-07 08:10:32',
    'updated_at': '2020-02-07 08:10:41',
    'task_execution_id': None
}

EXEC = executions.Execution(mock, EXEC_DICT)

SUB_WF_EXEC = executions.Execution(
    mock,
    {
        'id': '456',
        'workflow_id': '123e4567-e89b-12d3-a456-426655440000',
        'workflow_name': 'some_sub_wf',
        'workflow_namespace': '',
        'root_execution_id': 'ROOT_EXECUTION_ID',
        'description': '',
        'state': 'ERROR',
        'state_info': None,
        'created_at': '2020-02-07 08:10:32',
        'updated_at': '2020-02-07 08:10:41',
        'task_execution_id': 'abc'
    }
)

EX_RESULT = (
    '123',
    '123e4567-e89b-12d3-a456-426655440000',
    'some',
    '',
    '',
    '<none>',
    '<none>',
    'SUCCESS',
    None,
    '2020-02-07 08:10:32',
    '2020-02-07 08:10:41',
    '0:00:09'
)

SUB_WF_EX_RESULT = (
    '456',
    '123e4567-e89b-12d3-a456-426655440000',
    'some_sub_wf',
    '',
    '',
    'abc',
    'ROOT_EXECUTION_ID',
    'ERROR',
    None,
    '2020-02-07 08:10:32',
    '2020-02-07 08:10:41',
    '0:00:09'
)

EXECS_LIST = [EXEC, SUB_WF_EXEC]
EXEC_PUBLISHED = {"bar1": "val1", "var2": 2}
EXEC_WITH_PUBLISHED_DICT = EXEC_DICT.copy()
EXEC_WITH_PUBLISHED_DICT.update(
    {'published_global': jsonutils.dumps(EXEC_PUBLISHED)})
EXEC_WITH_PUBLISHED = executions.Execution(mock, EXEC_WITH_PUBLISHED_DICT)


class TestCLIExecutionsV2(base.BaseCommandTest):
    def setUp(self):
        super(TestCLIExecutionsV2, self).setUp()

    def tearDown(self):
        super(TestCLIExecutionsV2, self).tearDown()

    def test_create_wf_input_string(self):
        self.client.executions.create.return_value = EXEC

        result = self.call(
            execution_cmd.Create,
            app_args=['id', '{ "context": true }']
        )

        self.assertEqual(EX_RESULT, result[1])

    def test_create_wf_input_file(self):
        self.client.executions.create.return_value = EXEC

        path = pkg.resource_filename(
            'mistralclient',
            'tests/unit/resources/ctx.json'
        )

        result = self.call(
            execution_cmd.Create,
            app_args=['id', path]
        )

        self.assertEqual(EX_RESULT, result[1])

    def test_create_with_description(self):
        self.client.executions.create.return_value = EXEC

        result = self.call(
            execution_cmd.Create,
            app_args=['id', '{ "context": true }', '-d', '']
        )

        self.assertEqual(EX_RESULT, result[1])

    def test_update_state(self):
        states = ['RUNNING', 'SUCCESS', 'PAUSED', 'ERROR', 'CANCELLED']

        for state in states:
            self.client.executions.update.return_value = executions.Execution(
                mock,
                {
                    'id': '123',
                    'workflow_id': '123e4567-e89b-12d3-a456-426655440000',
                    'workflow_name': 'some',
                    'workflow_namespace': '',
                    'root_execution_id': '',
                    'description': '',
                    'state': state,
                    'state_info': None,
                    'created_at': '2020-02-07 08:10:32',
                    'updated_at': '2020-02-07 08:10:41',
                    'task_execution_id': None
                }
            )

            ex_result = list(EX_RESULT)
            ex_result[7] = state

            # We'll ignore "duration" since for not terminal states
            # it is unpredictable.
            del ex_result[11]
            ex_result = tuple(ex_result)

            result = self.call(
                execution_cmd.Update,
                app_args=['id', '-s', state]
            )

            result_ex = list(result[1])
            del result_ex[11]
            result_ex = tuple(result_ex)

            self.assertEqual(ex_result, result_ex)

    def test_update_invalid_state(self):
        states = ['IDLE', 'WAITING', 'DELAYED']

        for state in states:
            self.assertRaises(
                SystemExit,
                self.call,
                execution_cmd.Update,
                app_args=['id', '-s', state]
            )

    def test_resume_update_env(self):
        self.client.executions.update.return_value = EXEC

        result = self.call(
            execution_cmd.Update,
            app_args=['id', '-s', 'RUNNING', '--env', '{"k1": "foobar"}']
        )

        self.assertEqual(EX_RESULT, result[1])

    def test_update_description(self):
        self.client.executions.update.return_value = EXEC

        result = self.call(
            execution_cmd.Update,
            app_args=['id', '-d', 'foobar']
        )

        self.assertEqual(EX_RESULT, result[1])

    def test_list(self):
        self.client.executions.list.return_value = [SUB_WF_EXEC, EXEC]

        result = self.call(execution_cmd.List)

        self.assertEqual(
            [EX_RESULT, SUB_WF_EX_RESULT],
            result[1]
        )

    def test_sub_executions(self):
        self.client.executions.get_ex_sub_executions.return_value = \
            EXECS_LIST

        result = self.call(
            execution_cmd.SubExecutionsLister,
            app_args=[EXEC_DICT['id']]
        )

        self.assertEqual([EX_RESULT, SUB_WF_EX_RESULT], result[1])
        self.assertEqual(
            1,
            self.client.executions.get_ex_sub_executions.call_count
        )
        self.assertEqual(
            [mock.call(EXEC_DICT['id'], errors_only='', max_depth=-1)],
            self.client.executions.get_ex_sub_executions.call_args_list
        )

    def test_sub_executions_errors_only(self):
        self.client.executions.get_ex_sub_executions.return_value = \
            EXECS_LIST

        self.call(
            execution_cmd.SubExecutionsLister,
            app_args=[EXEC_DICT['id'], '--errors-only']
        )

        self.assertEqual(
            1,
            self.client.executions.get_ex_sub_executions.call_count
        )
        self.assertEqual(
            [mock.call(EXEC_DICT['id'], errors_only=True, max_depth=-1)],
            self.client.executions.get_ex_sub_executions.call_args_list
        )

    def test_sub_executions_with_max_depth(self):
        self.client.executions.get_ex_sub_executions.return_value = \
            EXECS_LIST

        self.call(
            execution_cmd.SubExecutionsLister,
            app_args=[EXEC_DICT['id'], '--max-depth', '3']
        )

        self.assertEqual(
            1,
            self.client.executions.get_ex_sub_executions.call_count
        )
        self.assertEqual(
            [mock.call(EXEC_DICT['id'], errors_only='', max_depth=3)],
            self.client.executions.get_ex_sub_executions.call_args_list
        )

    def test_list_with_pagination(self):
        self.client.executions.list.return_value = [EXEC]

        self.call(execution_cmd.List)
        self.client.executions.list.assert_called_once_with(
            fields=execution_cmd.ExecutionFormatter.fields(),
            limit=100,
            marker='',
            nulls='',
            sort_dirs='desc',
            sort_keys='created_at',
            task=None
        )

        self.call(
            execution_cmd.List,
            app_args=[
                '--oldest'
            ]
        )

        self.client.executions.list.assert_called_with(
            fields=execution_cmd.ExecutionFormatter.fields(),
            limit=100,
            marker='',
            nulls='',
            sort_keys='created_at',
            sort_dirs='asc',
            task=None
        )

        self.call(
            execution_cmd.List,
            app_args=[
                '--limit', '5',
                '--sort_keys', 'id, Workflow',
                '--sort_dirs', 'desc',
                '--marker', 'abc'
            ]
        )

        self.client.executions.list.assert_called_with(
            fields=execution_cmd.ExecutionFormatter.fields(),
            limit=5,
            marker='abc',
            nulls='',
            sort_keys='id, Workflow',
            sort_dirs='desc',
            task=None
        )

    def test_get(self):
        self.client.executions.get.return_value = EXEC

        result = self.call(execution_cmd.Get, app_args=['id'])

        self.assertEqual(EX_RESULT, result[1])

    def test_get_sub_wf_ex(self):
        self.client.executions.get.return_value = SUB_WF_EXEC

        result = self.call(execution_cmd.Get, app_args=['id'])

        self.assertEqual(SUB_WF_EX_RESULT, result[1])

    def test_delete(self):
        self.call(execution_cmd.Delete, app_args=['id'])

        self.client.executions.delete.assert_called_once_with(
            'id',
            force=False
        )

    def test_delete_with_force(self):
        self.call(execution_cmd.Delete, app_args=['id', '--force'])

        self.client.executions.delete.assert_called_once_with(
            'id',
            force=True
        )

    def test_delete_with_multi_names(self):
        self.call(execution_cmd.Delete, app_args=['id1', 'id2'])

        self.assertEqual(2, self.client.executions.delete.call_count)
        self.assertEqual(
            [mock.call('id1', force=False), mock.call('id2', force=False)],
            self.client.executions.delete.call_args_list
        )

    def test_get_published(self):
        self.client.executions.get.return_value = EXEC_WITH_PUBLISHED

        self.call(execution_cmd.GetPublished, app_args=['id'])

        self.assertDictEqual(
            EXEC_PUBLISHED,
            jsonutils.loads(self.app.stdout.write.call_args[0][0])
        )



# Copyright 2020 Nokia Software.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from oslo_utils import uuidutils

from mistralclient.api import base


class DynamicAction(base.Resource):
    resource_name = 'DynamicAction'


class DynamicActionManager(base.ResourceManager):
    resource_class = DynamicAction

    def get(self, identifier, namespace=''):
        self._ensure_not_empty(identifier=identifier)

        return self._get(
            '/dynamic_actions/%s?namespace=%s' % (identifier, namespace)
        )

    def create(self, name, class_name, code_source, scope='private',
               namespace=''):
        self._ensure_not_empty(
            name=name,
            class_name=class_name,
            code_source=code_source
        )

        data = {
            "name": name,
            "class_name": class_name,
            "scope": scope,
            "namespace": namespace
        }

        if uuidutils.is_uuid_like(code_source):
            data['code_source_id'] = code_source
        else:
            data['code_source_name'] = code_source

        return self._create('/dynamic_actions', data)

    def update(self, identifier, class_name=None, code_source=None,
               scope='private', namespace=''):
        self._ensure_not_empty(identifier=identifier)

        data = {
            'scope': scope,
            'namespace': namespace
        }

        if uuidutils.is_uuid_like(identifier):
            data['id'] = identifier
        else:
            data['name'] = identifier

        if class_name:
            data['class_name'] = class_name

        if code_source:
            if uuidutils.is_uuid_like(code_source):
                data['code_source_id'] = code_source
            else:
                data['code_source_name'] = code_source

        return self._update('/dynamic_actions', data)

    def list(self, marker='', limit=None, sort_keys='', sort_dirs='',
             fields='', namespace='', **filters):
        if namespace:
            filters['namespace'] = namespace

        query_string = self._build_query_params(
            marker=marker,
            limit=limit,
            sort_keys=sort_keys,
            sort_dirs=sort_dirs,
            fields=fields,
            filters=filters
        )

        return self._list(
            '/dynamic_actions%s' % query_string,
            response_key='dynamic_actions',
        )

    def delete(self, identifier, namespace=''):
        self._ensure_not_empty(identifier=identifier)

        self._delete(
            '/dynamic_actions/%s?namespace=%s' % (identifier, namespace)
        )



# Copyright 2014 Mirantis, Inc.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from unittest import mock

from mistralclient.api.v2 import event_triggers
from mistralclient.commands.v2 import event_triggers as event_triggers_cmd
from mistralclient.tests.unit import base


TRIGGER_DICT = {
    'id': '456',
    'name': 'my_trigger',
    'workflow_id': '123e4567-e89b-12d3-a456-426655440000',
    'workflow_input': {},
    'workflow_params': {},
    'exchange': 'dummy_exchange',
    'topic': 'dummy_topic',
    'event': 'event.dummy',
    'created_at': '1',
    'updated_at': '1'
}

TRIGGER = event_triggers.EventTrigger(mock, TRIGGER_DICT)


class TestCLITriggersV2(base.BaseCommandTest):

    @mock.patch('argparse.open', create=True)
    def test_create(self, mock_open):
        self.client.event_triggers.create.return_value = TRIGGER
        mock_open.return_value = mock.MagicMock(spec=open)

        result = self.call(
            event_triggers_cmd.Create,
            app_args=['my_trigger', '123e4567-e89b-12d3-a456-426655440000',
                      'dummy_exchange', 'dummy_topic', 'event.dummy',
                      '--params', '{}']
        )

        self.assertEqual(
            (
                '456', 'my_trigger', '123e4567-e89b-12d3-a456-426655440000',
                {}, 'dummy_exchange', 'dummy_topic', 'event.dummy', '1', '1'
            ),
            result[1]
        )

    def test_list(self):
        self.client.event_triggers.list.return_value = [TRIGGER]

        result = self.call(event_triggers_cmd.List)

        self.assertEqual(
            [(
                '456', 'my_trigger', '123e4567-e89b-12d3-a456-426655440000',
                {}, 'dummy_exchange', 'dummy_topic', 'event.dummy', '1', '1'
            )],
            result[1]
        )

    def test_get(self):
        self.client.event_triggers.get.return_value = TRIGGER

        result = self.call(event_triggers_cmd.Get, app_args=['id'])

        self.assertEqual(
            (
                '456', 'my_trigger', '123e4567-e89b-12d3-a456-426655440000',
                {}, 'dummy_exchange', 'dummy_topic', 'event.dummy', '1', '1'
            ),
            result[1]
        )

    def test_delete(self):
        self.call(event_triggers_cmd.Delete, app_args=['id'])

        self.client.event_triggers.delete.assert_called_once_with('id')

    def test_delete_with_multi_names(self):
        self.call(event_triggers_cmd.Delete, app_args=['id1', 'id2'])

        self.assertEqual(2, self.client.event_triggers.delete.call_count)
        self.assertEqual(
            [mock.call('id1'), mock.call('id2')],
            self.client.event_triggers.delete.call_args_list
        )



# Copyright 2014 - Mirantis, Inc.
# Copyright 2015 - StackStorm, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import logging
import os.path

from oslo_serialization import jsonutils

from osc_lib.command import command

from mistralclient.commands.v2 import base
from mistralclient.commands.v2 import executions
from mistralclient import utils

LOG = logging.getLogger(__name__)


class TaskFormatter(base.MistralFormatter):
    COLUMNS = [
        ('id', 'ID'),
        ('name', 'Name'),
        ('workflow_name', 'Workflow name'),
        ('workflow_namespace', 'Workflow namespace'),
        ('workflow_execution_id', 'Workflow Execution ID'),
        ('state', 'State'),
        ('state_info', 'State info'),
        ('created_at', 'Created at'),
        ('started_at', 'Started at'),
        ('finished_at', 'Finished at'),
        ('duration', 'Duration', True),
    ]

    @staticmethod
    def format(task=None, lister=False):
        if task:
            state_info = (task.state_info if not lister
                          else base.cut(task.state_info))

            duration = base.get_duration_str(task.started_at, task.finished_at)

            data = (
                task.id,
                task.name,
                task.workflow_name,
                task.workflow_namespace,
                task.workflow_execution_id,
                task.state,
                state_info,
                task.created_at,
                task.started_at or '<none>',
                task.finished_at or '<none>',
                duration
            )
        else:
            data = (tuple('' for _ in range(len(TaskFormatter.COLUMNS))),)

        return TaskFormatter.headings(), data


class List(base.MistralExecutionLister):
    """List all tasks."""

    def get_parser(self, prog_name):
        parser = super(List, self).get_parser(prog_name)

        parser.add_argument(
            'workflow_execution',
            nargs='?',
            help='Workflow execution ID associated with list of Tasks.'
        )
        return parser

    def _get_format_function(self):
        return TaskFormatter.format_list

    def _get_resources(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine

        return mistral_client.tasks.list(
            parsed_args.workflow_execution,
            marker=parsed_args.marker,
            limit=parsed_args.limit,
            sort_keys=parsed_args.sort_keys,
            sort_dirs=parsed_args.sort_dirs,
            fields=TaskFormatter.fields(),
            **base.get_filters(parsed_args)
        )


class Get(command.ShowOne):
    """Show specific task."""

    def get_parser(self, prog_name):
        parser = super(Get, self).get_parser(prog_name)

        parser.add_argument('task', help='Task identifier')

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine
        execution = mistral_client.tasks.get(parsed_args.task)

        return TaskFormatter.format(execution)


class GetResult(command.Command):
    """Show task output data."""

    def get_parser(self, prog_name):
        parser = super(GetResult, self).get_parser(prog_name)
        parser.add_argument(
            'id',
            help='Task ID')

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine
        result = mistral_client.tasks.get(parsed_args.id).result

        try:
            result = jsonutils.loads(result)
            result = jsonutils.dumps(result, indent=4) + "\n"
        except Exception:
            LOG.debug("Task result is not JSON.")

        self.app.stdout.write(result or "\n")


class GetPublished(command.Command):
    """Show task published variables."""

    def get_parser(self, prog_name):
        parser = super(GetPublished, self).get_parser(prog_name)
        parser.add_argument(
            'id',
            help='Task ID')

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine
        res = mistral_client.tasks.get(parsed_args.id)
        published = res.published
        published_glob = res.published_global \
            if hasattr(res, 'published_global') else None

        try:
            published = jsonutils.loads(published)
            published = jsonutils.dumps(published, indent=4) + "\n"

            if published_glob:
                published_glob = jsonutils.loads(published_glob)
                published += jsonutils.dumps(published_glob, indent=4) + "\n"
        except Exception:
            LOG.debug("Task result is not JSON.")

        self.app.stdout.write(published or "\n")


class Rerun(command.ShowOne):
    """Rerun an existing task."""

    def get_parser(self, prog_name):
        parser = super(Rerun, self).get_parser(prog_name)

        parser.add_argument(
            'id',
            help='Task identifier'
        )

        parser.add_argument(
            '--resume',
            action='store_true',
            dest='resume',
            default=False,
            help=('rerun only failed or unstarted action '
                  'executions for with-items task')
        )

        parser.add_argument(
            '-e',
            '--env',
            dest='env',
            help='Environment variables'
        )

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine

        env = (
            utils.load_file(parsed_args.env)
            if parsed_args.env and os.path.isfile(parsed_args.env)
            else utils.load_content(parsed_args.env)
        )

        execution = mistral_client.tasks.rerun(
            parsed_args.id,
            reset=(not parsed_args.resume),
            env=env
        )

        return TaskFormatter.format(execution)


class SubExecutionsLister(executions.SubExecutionsBaseLister):

    def _get_format_function(self):
        return executions.ExecutionFormatter.format_list

    def _get_resources_function(self):
        mistral_client = self.app.client_manager.workflow_engine

        return mistral_client.tasks.get_task_sub_executions



# Copyright 2014 - Mirantis, Inc.
# Copyright 2015 - StackStorm, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from unittest import mock


from mistralclient.api.v2 import workflows
from mistralclient.commands.v2 import base as cmd_base
from mistralclient.commands.v2 import workflows as workflow_cmd
from mistralclient.tests.unit import base


WORKFLOW_DICT = {
    'id': '1-2-3-4',
    'name': 'a',
    'namespace': '',
    'project_id': '12345',
    'tags': ['a', 'b'],
    'input': 'param',
    'scope': 'private',
    'created_at': '1',
    'updated_at': '1'
}

WF_DEF = """
version: '2.0'

flow:
  tasks:
    task1:
      action: nova.servers_get server="1"
"""

WF_WITH_DEF_DICT = WORKFLOW_DICT.copy()
WF_WITH_DEF_DICT.update({'definition': WF_DEF})
WORKFLOW = workflows.Workflow(mock, WORKFLOW_DICT)
WORKFLOW_WITH_DEF = workflows.Workflow(mock, WF_WITH_DEF_DICT)


class TestCLIWorkflowsV2(base.BaseCommandTest):
    @mock.patch('argparse.open', create=True)
    def test_create(self, mock_open):
        self.client.workflows.create.return_value = [WORKFLOW]

        result = self.call(workflow_cmd.Create, app_args=['1.txt'])

        self.assertEqual(
            [('1-2-3-4', 'a', '', '12345', 'a, b', 'param', 'private',
              '1', '1')],
            result[1]
        )

    @mock.patch('argparse.open', create=True)
    def test_create_public(self, mock_open):
        wf_public_dict = WORKFLOW_DICT.copy()
        wf_public_dict['scope'] = 'public'
        workflow_public = workflows.Workflow(mock, wf_public_dict)
        self.client.workflows.create.return_value = [workflow_public]

        result = self.call(
            workflow_cmd.Create,
            app_args=['1.txt', '--public']
        )

        self.assertEqual(
            [('1-2-3-4', 'a', '', '12345', 'a, b', 'param', 'public',
              '1', '1')],
            result[1]
        )

        self.assertEqual(
            'public',
            self.client.workflows.create.call_args[1]['scope']
        )

    @mock.patch('argparse.open', create=True)
    def test_create_long_input(self, mock_open):
        wf_long_input_dict = WORKFLOW_DICT.copy()
        long_input = ', '.join(
            ['var%s' % i for i in range(10)]
        )
        wf_long_input_dict['input'] = long_input
        workflow_long_input = workflows.Workflow(mock, wf_long_input_dict)
        self.client.workflows.create.return_value = [workflow_long_input]

        result = self.call(workflow_cmd.Create, app_args=['1.txt'])

        self.assertEqual(
            [('1-2-3-4', 'a', '', '12345', 'a, b', cmd_base.cut(long_input),
              'private', '1', '1')],
            result[1]
        )

    @mock.patch('argparse.open', create=True)
    def test_update(self, mock_open):
        self.client.workflows.update.return_value = [WORKFLOW]

        result = self.call(workflow_cmd.Update, app_args=['1.txt'])

        self.assertEqual(
            [('1-2-3-4', 'a', '', '12345', 'a, b', 'param', 'private',
              '1', '1')],
            result[1]
        )

    @mock.patch('argparse.open', create=True)
    def test_update_public(self, mock_open):
        self.client.workflows.update.return_value = [WORKFLOW]

        result = self.call(
            workflow_cmd.Update,
            app_args=['1.txt', '--public']
        )

        self.assertEqual(
            [('1-2-3-4', 'a', '', '12345', 'a, b', 'param', 'private',
              '1', '1')],
            result[1]
        )

        self.assertEqual(
            'public',
            self.client.workflows.update.call_args[1]['scope']
        )

    @mock.patch('argparse.open', create=True)
    def test_update_with_id(self, mock_open):
        self.client.workflows.update.return_value = WORKFLOW

        result = self.call(
            workflow_cmd.Update,
            app_args=['1.txt', '--id', '1-2-3-4']
        )

        self.assertEqual(
            [('1-2-3-4', 'a', '', '12345', 'a, b', 'param', 'private',
              '1', '1')],
            result[1]
        )

    def test_list(self):
        self.client.workflows.list.return_value = [WORKFLOW]

        result = self.call(workflow_cmd.List)

        self.assertEqual(
            [('1-2-3-4', 'a', '', '12345', 'a, b', 'param', 'private',
              '1', '1')],
            result[1]
        )

    def test_get(self):
        self.client.workflows.get.return_value = WORKFLOW

        result = self.call(workflow_cmd.Get, app_args=['name'])

        self.assertEqual(
            ('1-2-3-4', 'a', '', '12345', 'a, b', 'param', 'private',
             '1', '1'),
            result[1]
        )

    def test_delete(self):
        self.call(workflow_cmd.Delete, app_args=['name'])

        self.client.workflows.delete.assert_called_once_with('name', None)

    def test_delete_with_multi_names(self):
        self.call(workflow_cmd.Delete, app_args=['name1', 'name2'])

        self.assertEqual(2, self.client.workflows.delete.call_count)
        self.assertEqual(
            [mock.call('name1', None), mock.call('name2', None)],
            self.client.workflows.delete.call_args_list
        )

    def test_get_definition(self):
        self.client.workflows.get.return_value = WORKFLOW_WITH_DEF

        self.call(workflow_cmd.GetDefinition, app_args=['name'])

        self.app.stdout.write.assert_called_with(WF_DEF)

    @mock.patch('argparse.open', create=True)
    def test_validate(self, mock_open):
        self.client.workflows.validate.return_value = {'valid': True}

        result = self.call(workflow_cmd.Validate, app_args=['wf.yaml'])

        self.assertEqual((True, None), result[1])

    @mock.patch('argparse.open', create=True)
    def test_validate_failed(self, mock_open):
        self.client.workflows.validate.return_value = {
            'valid': False,
            'error': 'Invalid DSL...'
        }

        result = self.call(workflow_cmd.Validate, app_args=['wf.yaml'])

        self.assertEqual((False, 'Invalid DSL...'), result[1])



#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
import mistralclient.auth.keystone
from mistralclient.tests.unit.v2 import base


class TestKeystone(base.BaseClientV2Test):
    def setUp(self):
        super(TestKeystone, self).setUp()
        self.keystone = mistralclient.auth.keystone.KeystoneAuthHandler()

    def test_get_auth_token(self):
        auth = self.keystone._get_auth(
            auth_token="token",
            auth_url="url",
            project_id="project_id",
        )

        self.assertEqual("url", auth.auth_url)
        elements = auth.get_cache_id_elements()
        self.assertIsNotNone(elements["token"])
        self.assertIsNotNone(elements["project_id"])

    def test_remove_domain(self):
        params = {
            "param1": "p",
            "target_param2": "p2",
            "user_domain_param3": "p3",
            "target_project_domain_param4": "p4"
        }
        dedomained = self.keystone._remove_domain(params)
        self.assertIn("param1", dedomained)
        self.assertIn("target_param2", dedomained)
        self.assertNotIn("user_domain_param3", dedomained)
        self.assertNotIn("target_project_domain_param4", dedomained)

    def test_separate_target_reqs(self):
        params = {
            "a": 1,
            "target_b": 2,
            "c": 3,
            "target_d": 4,
            "target_target": 5,
            "param_target": 6
        }
        nontarget, target = self.keystone._separate_target_reqs(params)
        self.assertIn("a", nontarget)
        self.assertIn("c", nontarget)
        self.assertIn("param_target", nontarget)
        self.assertIn("b", target)
        self.assertIn("d", target)
        self.assertIn("target", target)

    def test_verify(self):
        self.assertTrue(self.keystone._verification_needed("", False))
        self.assertFalse(self.keystone._verification_needed("", True))
        self.assertFalse(self.keystone._verification_needed("cert", True))
        self.assertEqual(
            self.keystone._verification_needed("cert", False),
            "cert"
        )
